#define _GLOBAL
// NOTICE! 
// "exe/Shaders/Globals.hlsli" IS OWERWRITTEN BY "source/Shaders/Globals.hlsli" AT COMPILE TIME!
// CHANGES TO THIS FILE MUST BE MADE IN "source/Shaders/Globals.hlsli" OR THEY WILL BE LOST!

//Useful defines
#define PI              3.1415926535897932384
#define TO_RAD          0.0174532925199432957
#define TO_DEG          57.295779513082320876
#define EULER           2.7182818284590452353
#define ROOT_TWO        1.4142135623730950488
#define GOLDEN_RATIO    1.6180339887498948482
#define EPSILON         float(1e-10)

//Color functions
float3 RGBtoHCV(in float3 RGB)
{
    // Based on work by Sam Hocevar and Emil Persson
    float4 P = (RGB.g < RGB.b) ? float4(RGB.bg, -1.0, 2.0 / 3.0) : float4(RGB.gb, 0.0, -1.0 / 3.0);
    float4 Q = (RGB.r < P.x) ? float4(P.xyw, RGB.r) : float4(RGB.r, P.yzx);
    float C = Q.x - min(Q.w, Q.y);
    float H = abs((Q.w - Q.y) / (6 * C + EPSILON) + Q.z);
    return float3(H, C, Q.x);
}

float3 HUEtoRGB(in float H)
{
    float R = abs(H * 6 - 3) - 1;
    float G = 2 - abs(H * 6 - 2);
    float B = 2 - abs(H * 6 - 4);
    return saturate(float3(R, G, B));
}

float3 HSVtoRGB(in float3 HSV)
{
    float3 RGB = HUEtoRGB(HSV.x);
    return ((RGB - 1) * HSV.y + 1) * HSV.z;
}

float3 HSLtoRGB(in float3 HSL)
{
    float3 RGB = HUEtoRGB(HSL.x);
    float C = (1 - abs(2 * HSL.z - 1)) * HSL.y;
    return (RGB - 0.5) * C + HSL.z;
}

float3 RGBtoHSV(in float3 RGB)
{
    float3 HCV = RGBtoHCV(RGB);
    float S = HCV.y / (HCV.z + EPSILON);
    return float3(HCV.x, S, HCV.z);
}

float3 RGBtoHSL(in float3 RGB)
{
    float3 HCV = RGBtoHCV(RGB);
    float L = HCV.z - HCV.y / 2;
    float S = HCV.y / (1 - abs(L * 2 - 1) + EPSILON);
    return float3(HCV.x, S, L);
}

//The global framebuffer object
cbuffer FrameBuffer : register(b0)
{
    float4x4 toCamera;
    float4x4 toProjection;
    float4x4 fromProjection;
    float4x4 fromCamera;
    float4 cameraPos;
    uint screenWidth;
    uint screenHeight;
    float totalTime;
    float projectionA;
    float projectionB;
}

//All available samplers
SamplerState defaultWrapSampler : register(s0);
SamplerState pointWrapSampler : register(s1);
SamplerState anisotropicWrapSampler : register(s2);
SamplerState defaultClampSampler : register(s3);
SamplerState pointClampSampler : register(s4);
SamplerState anisotropicClampSampler : register(s5);

//constant textures
Texture2D randomTexture : register(t10);
Texture3D LUTTexture : register(t11);

// Space conversion
float2 PixelSpaceToScreenSpace(float2 aPos)
{
    float2 result = aPos / float2(screenWidth, screenHeight);
    result *= 2.0f;
    return result;
}