float4 quat_conj(float4 aQuat)
{
    return float4(-aQuat.xyz, aQuat.w);
}

float3 quat_mul(float4 aQuat, float3 aVector)
{
    float4 v = aQuat;
    const float num = v.x * 2.0f;
    const float num2 = v.y * 2.0f;
    const float num3 = v.z * 2.0f;
    const float num4 = v.x * num;
    const float num5 = v.y * num2;
    const float num6 = v.z * num3;
    const float num7 = v.x * num2;
    const float num8 = v.x * num3;
    const float num9 = v.y * num3;
    const float num10 = v.w * num;
    const float num11 = v.w * num2;
    const float num12 = v.w * num3;
    float3 result;
    result.x = (1.0f - (num5 + num6)) * aVector.x + (num7 - num12) * aVector.y + (num8 + num11) * aVector.z;
    result.y = (num7 + num12) * aVector.x + (1.f - (num4 + num6)) * aVector.y + (num9 - num10) * aVector.z;
    result.z = (num8 - num11) * aVector.x + (num9 + num10) * aVector.y + (1.f - (num4 + num5)) * aVector.z;
    return result;
}

float4 quat_from_matrix(float4x4 aRotationMatrix)
{
    float4x4 m = aRotationMatrix;
    float4 res;

    float trace = m._11 + m._22 + m._33;
    if (trace > 0.f)
    {
        float s = sqrt(trace + 1.0f);
        res.w = s / 2.0f;
        s = 0.5f / s;
            
        res.x = (m._23 - m._32) * s;
        res.y = (m._31 - m._13) * s;
        res.z = (m._12 - m._21) * s;
        return res;
    }
    else
    {
        uint i = 0;
        if (m._22 > m._11)
        {
            i = 1;
        }
        if (m._33 > m[i][i])
        {
            i = 2;
        }

        uint next[3] = { 1, 2, 0 };
        uint j = next[i];
        uint k = next[j];

        float s = sqrt(m[i][i] - (m[j][j] + m[k][k]) + 1.0f);
        float q[4];
        q[i] = s * 0.5f;
        if (s != 0.0f)
        {
            s = 0.5f / s;
        }
        q[3] = (m[j][k] - m[k][j]) * s;
        q[j] = (m[i][j] + m[j][i]) * s;
        q[k] = (m[i][k] + m[k][i]) * s;

        res.x = q[0];
        res.y = q[1];
        res.z = q[2];
        res.w = q[3];
    }
}