#include "../Globals.hlsli"
#include "../ModelShaderStructs.hlsli"

cbuffer ObjectBuffer : register(b1)
{
    float4x4 toWorld;
}

Texture2D bumpTex : register(t2);

// Called once per tessellated vertex
[domain("tri")]
VertexToPixel DS(TriTess input, float3 uvw : SV_DomainLocation, const OutputPatch<HullOut, 3> TrianglePatch)
{
    VertexToPixel Out;
    float4x4 mvp = mul(toProjection, toCamera);
	
    // Interpolate all the values.
    float3 position = TrianglePatch[0].myWorldPosition.xyz * uvw.x + TrianglePatch[1].myWorldPosition.xyz * uvw.y + TrianglePatch[2].myWorldPosition.xyz * uvw.z;
    float3 normal = TrianglePatch[0].myNormal.xyz * uvw.x + TrianglePatch[1].myNormal.xyz * uvw.y + TrianglePatch[2].myNormal.xyz * uvw.z;
    float3 tangent = TrianglePatch[0].myTangent.xyz * uvw.x + TrianglePatch[1].myTangent.xyz * uvw.y + TrianglePatch[2].myTangent.xyz * uvw.z;
    float3 binormal = TrianglePatch[0].myBinormal.xyz * uvw.x + TrianglePatch[1].myBinormal.xyz * uvw.y + TrianglePatch[2].myBinormal.xyz * uvw.z;
    float4 color = TrianglePatch[0].myColor.rgba * uvw.x + TrianglePatch[1].myColor.rgba * uvw.y + TrianglePatch[2].myColor.rgba * uvw.z;
    float2 uv = TrianglePatch[0].myUV.xy * uvw.x + TrianglePatch[1].myUV.xy * uvw.y + TrianglePatch[2].myUV.xy * uvw.z;

    // sample the displacement map for the magnitude of displacement
    float fDisplacement = bumpTex.SampleLevel(defaultClampSampler, uv, 0).a * 2.0f - 1.0f;
    fDisplacement *= 0.2f; //scale
    fDisplacement += 0; //bias
    float3 vDirection = normal.xyz; // direction is opposite normal
    // translate the position
    position += vDirection * fDisplacement;
    // transform to clip space

    Out.myPosition = mul(mvp, float4(position, 1.0f));
    Out.myWorldPosition = float4(position, 1.0f);
    Out.myNormal = float4(normal, 0);
    Out.myTangent = float4(tangent, 0);
    Out.myBinormal = float4(binormal, 0);
    Out.myUV = uv;
    Out.myColor = color;

    return Out;
}