#include "..\Globals.hlsli"
#include "..\ParticleShaderStructs.hlsli"

static const float2 uv[4] =
{
    { 0.0f, 0.0f },
    { 1.0f, 0.0f },
    { 0.0f, 1.0f },
    { 1.0f, 1.0f }
};

float3 ExtrudeDirection(float3 A, float3 B, float3 C)
{
    float3 AC = normalize(C - A);
    float3 AB = B - A;
    float scalar = dot(AB, AC);
    float3 P = A + AC * scalar;
    float3 extrudeDirection = P - B;
    extrudeDirection.y += 0.01; // in case P == B
    extrudeDirection = normalize(extrudeDirection);

    return extrudeDirection;
}

float3 TriNormal(float3 A, float3 B, float3 C)
{
    return cross(B - A, C - A);
}

[MaxVertexCount(6)]
void main(lineadj VertexToGeometry input[4], inout TriangleStream<GeometryToPixel> output)
{
    float3 dir[2] =
    {
        ExtrudeDirection(input[0].myPosition.xyz, input[1].myPosition.xyz, input[2].myPosition.xyz),
        ExtrudeDirection(input[1].myPosition.xyz, input[2].myPosition.xyz, input[3].myPosition.xyz)
    };

    float4 positions[4] =
    {
        input[1].myPosition + float4(dir[0] * 0.5f * input[1].mySize.x, 0),
        input[2].myPosition + float4(dir[1] * 0.5f * input[2].mySize.x, 0),
        input[1].myPosition - float4(dir[0] * 0.5f * input[1].mySize.x, 0),
        input[2].myPosition - float4(dir[1] * 0.5f * input[2].mySize.x, 0)
    };

    for (uint i = 0; i < 2; ++i)
    {
        uint3 vertexOrder = uint3(0 + i, 1 + i * 2, 2);

        float3 triNormal = TriNormal(positions[vertexOrder[0]].xyz, positions[vertexOrder[1]].xyz, positions[vertexOrder[2]].xyz);
        float3 triToEye = -(positions[vertexOrder[0]] + positions[vertexOrder[1]] + positions[vertexOrder[2]]) / 3.0;

        if (dot(triNormal, triToEye) < 0)
        {
            vertexOrder.xz = vertexOrder.zx;
        }

        for (uint j = 0; j < 3; ++j)
        {
            GeometryToPixel vertex;
            uint vi = vertexOrder[j];
            uint n = vi % 2;
            vertex.myPosition = mul(toProjection, positions[vi]);
            vertex.myColor = input[n + 1].myColor;
            vertex.myUV = uv[vi];
            vertex.myRandom = input[n + 1].myRandom;
            vertex.myLifeTime = input[n + 1].myLifeTime;
            vertex.myInitialLifeTime = input[n + 1].myInitialLifeTime;
            output.Append(vertex);
        }
        output.RestartStrip();
    }
}