#include "..\Globals.hlsli"
#include "..\ParticleShaderStructs.hlsli"
#include "..\Quaternion.hlsli"

static const float2 offset[4] =
{
    { -0.5f, 0.5f },
    { 0.5f, 0.5f },
    { -0.5f, -0.5f },
    { 0.5f, -0.5f }
};

static const float2 uv[4] =
{
    { 0.0f, 0.0f },
    { 1.0f, 0.0f },
    { 0.0f, 1.0f },
    { 1.0f, 1.0f }
};

[MaxVertexCount(4)]
void main(point VertexToGeometry input[1], inout TriangleStream<GeometryToPixel> output)
{
    VertexToGeometry inputvertex = input[0];    
    
    float c = cos(inputvertex.myQuadRotation);
    float s = sin(inputvertex.myQuadRotation);
    float3x3 rotation =
    {
        c, s, 0,
        -s, c, 0,
        0, 0, 1
    };

    for (uint index = 0; index < 4; ++index)
    {
        float3 pos = float3(offset[index] * inputvertex.mySize, 0);
        pos = mul(rotation, pos);
        pos = quat_mul(inputvertex.myWorldRotation, pos);
        float4 objectPos = float4(pos + inputvertex.myPosition.xyz, 1);
        float4 viewPos = mul(toCamera, objectPos);
        float4 projectionPos = mul(toProjection, viewPos);

        GeometryToPixel vertex;
        vertex.myPosition = projectionPos;

        vertex.myColor = inputvertex.myColor;
        vertex.myUV = uv[index];
        vertex.myRandom = inputvertex.myRandom;
        vertex.myLifeTime = inputvertex.myLifeTime;
        vertex.myInitialLifeTime = inputvertex.myInitialLifeTime;

        output.Append(vertex);
    }
}