#include "..\Globals.hlsli"
#include "..\GBufferOutput.hlsli"
#include "..\ModelShaderStructs.hlsli"

Texture2D colorTex : register(t0);          // RGB = albedo,    A = Alpha
Texture2D surfTex : register(t1);           // R   = AO,        G = roughness, B = metallic, A = blending mask
Texture2D bumpTex : register(t2);           // RGB = normal,    A = height
Texture2D emitTex : register(t3);           // RBG = emissive,  A = scattering

GBufferOutput main(VertexToPixel input)
{
    float4 bump = bumpTex.Sample(defaultClampSampler, input.myUV.xy);
    float3 normal = bump.rgb * 2 - 1;
    bool hasNormal = !(bump.r == 0 && bump.g == 0 && bump.b == 0);
    normal = hasNormal * normalize(normal) + !hasNormal * float3(0, 0, 1);

    float3x3 tangentspace = transpose(float3x3(normalize(input.myTangent.xyz), normalize(input.myBinormal.xyz), normalize(input.myNormal.xyz)));
    normal = mul(tangentspace, normal);

    GBufferOutput output;
    output.myColor = colorTex.Sample(defaultClampSampler, input.myUV.xy);
    output.mySurf = surfTex.Sample(defaultClampSampler, input.myUV.xy);
    output.myBump = float4(normal, bump.a);
    output.myEmissive = emitTex.Sample(defaultClampSampler, input.myUV.xy);
    return output;

}