#include "..\Globals.hlsli"
#include "..\GBufferInput.hlsli"

cbuffer LightBuffer : register(b2)
{
    float4 myToDirectionalLight;
    float3 myDirectionalLightColor;
}
// Taken from Norrsken PBR ShaderFX shader
float DistributionGGX(float roughness, float3 viewVector, float3 normalVector, float3 lightVector) 
{ 
    float3 halfVector = normalize(viewVector + lightVector);
	
    float roughness2 = roughness * roughness;
	float roughness4 = roughness2 * roughness2;
	
	float NdH = max(dot(normalVector, halfVector), 0);
	float NdH2 = NdH * NdH;

 	float denominator = (NdH2 * (roughness2 - 1.0) + 1.0);
 	denominator = 3.14159265368979 * denominator * denominator;

	return roughness2 / denominator;
}
// Taken from Norrsken PBR ShaderFX shader
float GeometryGGX(float roughness, float3 viewVector, float3 normalVector, float3 lightVector) 
{ 
	float constant = roughness + 1;
	constant = (constant*constant) / 8;
	
	float numberator1 = max(dot(normalVector, viewVector), 0);
	float numberator2 = max(dot(normalVector, lightVector), 0);
	float denominator1 = numberator1 * (1 - constant) + constant;
	float denominator2 = numberator2 * (1 - constant) + constant;

	return (numberator1 * numberator2) / (denominator1 * denominator2);
}
// Taken from Norrsken PBR ShaderFX shader
float3 FresnelGGX(float3 albedo, float metallic, float3 viewVector, float3 normalVector, float3 lightVector) 
{ 
	float3 F0 = lerp(float3(0.04, 0.04, 0.04), albedo, metallic);

	viewVector = normalize(viewVector);	// ShaderFX normalize doesn't cut it for some reason...
	lightVector = normalize(lightVector);	// ShaderFX normalize doesn't cut it for some reason...

	float HdV = max(dot(normalize(viewVector + lightVector), viewVector), 0);

	return F0 + (1 - F0) * pow(1 - HdV, 5); 
}
// Taken from Norrsken PBR ShaderFX shader
PixelOutput main(VertexToPixel input)
{
    float3 normalVector = normalize(Normal(input));
	float3 viewVector = normalize(cameraPos.xyz - WorldPos(input).xyz);
	float3 lightVector = myToDirectionalLight.xyz;
	float3 lightRadience = myDirectionalLightColor.rgb;
	
	float3 albedo = Albedo(input).rgb;
    float roughness = (Roughness(input) - 0.5) * 2;
	
	//roughness = pow(roughness, 2.2);
	
	float metallic = Metalness(input);

	viewVector = normalize(viewVector);
	normalVector = normalize(normalVector);
	lightVector = normalize(lightVector);

    float distribution = DistributionGGX(roughness, viewVector, normalVector, lightVector);
	float geometry = GeometryGGX(roughness, viewVector, normalVector, lightVector);
	float3 fresnel = FresnelGGX(albedo, metallic, viewVector, normalVector, lightVector);

	float denominator = 4 * max(dot(normalVector, viewVector), 0) * max(dot(normalVector, lightVector), 0);
    float3 specular = (distribution * geometry * fresnel) / max(denominator, 0.001);

    float aspectRatio = float(screenWidth) / float(screenHeight);
    float pattern = (input.myUV.g / aspectRatio) + input.myUV.r;
    float patternScale = 0.3f;
    pattern *= 255;
    pattern = fmod(pattern, patternScale);
    pattern = (pattern / patternScale) * 2.0 - 1.0f;
    pattern *= 0.5f;

	float3 diffuseFactor = float3(1, 1, 1) - fresnel;
	diffuseFactor *= 1 - metallic;	  
	float NdL = max(dot(normalVector, lightVector), 0);
    float3 radiance = (diffuseFactor * albedo / 3.14159265358979 + specular) * lightRadience * NdL;
	
    PixelOutput returnValue;
    radiance.rgb *= 255.0;
    radiance.rgb += pattern;
    radiance.rgb /= 255.0;
    float3 c = radiance;
    returnValue.myColor.rgb = c;
    returnValue.myColor.a = 1 * IsNotEmpty(input);
    return returnValue;
}