#include "..\Globals.hlsli"
#include "..\GBufferInput.hlsli"

TextureCube cubeMapTexture : register(t20);

cbuffer LightBuffer : register(b2)
{
    float myDiffuseIntensity, mySpecularIntensity;
    float trash[5];
    uint myCubeMapMipCount;
}

static const float k0 = 0.00098f;
static const float k1 = 0.9921f;
static const float fakeLysMapSpecularPower = (2.0f / (0.0014f * 0.0014f)) - 2.0f;
static const float fMaxT = (exp2(-10.f / sqrt(fakeLysMapSpecularPower)) - k0) / k1;

float GetSpecPowToMip(float aSpecPow, uint aMipCount)
{
    float fSmultMaxT = (exp2(-10.f / sqrt(aSpecPow)) - k0) / k1;
    return float(aMipCount - 1 - 0) * (1.0f - clamp(fSmultMaxT / fMaxT, 0.0f, 1.0f));
}

float RoughToSPow(float aRoughness)
{
    return  (2.0f / (aRoughness * aRoughness)) - 2.0f;
}
/*
float3 AmbientSpec(VertexToPixel input, float roughness)
{
    float3 normal = Normal(input).xyz;

    float3 toEye = normalize(cameraPos.xyz - WorldPos(input).xyz);
    float3 reflectionVector = -reflect(toEye, normal);

    float fakeLysSpecularPower = roughness;
	float specMip = roughness * (myCubeMapMipCount - 1);

    float lysMipMap = GetSpecPowToMip(fakeLysSpecularPower, specMip);
	roughness = 1;
	
    return cubeMapTexture.SampleLevel(defaultWrapSampler, reflectionVector,pow(roughness, 0.5) * (10)).rgb * 4;
}*/
/*
float3 AmbientSpec(VertexToPixel input, float roughness)
{
	roughness = pow(roughness, 0.5);
	
    float3 normal = normalize(Normal(input));
	
	float3 toEye = normalize(cameraPos.xyz - WorldPos(input).xyz);
    normal = -reflect(toEye, normal);
	
	float3 vectorDw_x = float3(0.96194, -0.03806, -0.270598);
	float3 vectorDw_y = float3(-0.03806, 0.96194, -0.270598);
	float3 vectorDw_z = float3(0.270598, 0.270598, 0.92388); // Z is up
	float3x3 matrixDw = float3x3(vectorDw_x.xzy, vectorDw_z.xzy, vectorDw_y.xzy);		

	float3 vectorUp_x = float3(0.96194, -0.03806, 0.270598);
	float3 vectorUp_y = float3(-0.03806, 0.96194, 0.270598);
	float3 vectorUp_z = float3(-0.270598, -0.270598, 0.92388); // Z is up
	float3x3 matrixUp = float3x3(vectorDw_x.xzy, vectorDw_z.xzy, vectorDw_y.xzy);		
	
	float3 normalDw = normalize(lerp(normal, mul(matrixDw, normal), roughness));
	float3 normalUp = normalize(lerp(normal, mul(matrixUp, normal), roughness));
	

	float mipLevel = roughness * 10;

	float3 blurredCubemap = cubeMapTexture.SampleLevel(defaultWrapSampler, normal, mipLevel).rgb;
	blurredCubemap += cubeMapTexture.SampleLevel(defaultWrapSampler, normalDw, mipLevel).rgb;
	blurredCubemap += cubeMapTexture.SampleLevel(defaultWrapSampler, normalUp, mipLevel).rgb;
	
	blurredCubemap *= 1.333333;
    return blurredCubemap;
}*/

float3 AmbientSpec(VertexToPixel input, float roughness)
{
    float3 normal = normalize(Normal(input));
	
	float3 toEye = normalize(cameraPos.xyz - WorldPos(input).xyz);
    normal = -reflect(toEye, normal);

	float3 vectorA_x = float3(0.793893, -0.206107, -0.572062);
	float3 vectorA_y = float3(-0.572061, 0.793893, -0.206107);
	float3 vectorA_z = float3(0.572061, 0.572061, 0.587785); // Z is up
	float3x3 matrixA = float3x3(vectorA_x.xzy, vectorA_z.xzy, vectorA_y.xzy);	
	float3 normalA = normalize(mul(matrixA, normal));
		
	float3 vectorB_x = float3(0.707107, -0.707107, 0);
	float3 vectorB_y = float3(0.572062, 0.572061, 0.587786);
	float3 vectorB_z = float3(-0.415627, -0.415627, 0.809017); // Z is up
	float3x3 matrixB = float3x3(vectorB_x.xzy, vectorB_z.xzy, vectorB_y.xzy);	
	float3 normalB = normalize(mul(matrixB, normal));
		
	float3 vectorC_x = float3(0.572061, 0.572062, 0.587785);
	float3 vectorC_y = float3(-0.707107,0.707107, 0);
	float3 vectorC_z = float3(-0.415627, -0.415627, 0.809017); // Z is up
	float3x3 matrixC = float3x3(vectorC_x.xzy, vectorC_z.xzy, vectorC_y.xzy);	
	float3 normalC = normalize(mul(matrixA, normal));

	float normalOffsetAmplitude = roughness;
	
	normalA = normalize(lerp(normal, normalA, normalOffsetAmplitude));
	normalB = normalize(lerp(normal, normalB, normalOffsetAmplitude));
	normalC = normalize(lerp(normal, normalC, normalOffsetAmplitude));
	
    float mipLevel = pow(roughness, 0.5) * 10;

	float3 cubemapA = cubeMapTexture.SampleLevel(defaultWrapSampler, normalA, mipLevel).rgb;
	float3 cubemapB = cubeMapTexture.SampleLevel(defaultWrapSampler, normalB, mipLevel).rgb;
	float3 cubemapC = cubeMapTexture.SampleLevel(defaultWrapSampler, normalC, mipLevel).rgb;
	
	float3 blurredCubemap = (cubemapA + cubemapB + cubemapC) / 3;
    return blurredCubemap * 4;
}


float3 AmbientDiffuse(VertexToPixel input)
{
    float3 normal = normalize(Normal(input));
/*
	float3 vectorA_x = float3(0.793893, -0.206107, -0.572062);
	float3 vectorA_y = float3(-0.572061, 0.793893, -0.206107);
	float3 vectorA_z = float3(0.572061, 0.572061, 0.587785); // Z is up
	float3x3 matrixA = float3x3(vectorA_x.xzy, vectorA_z.xzy, vectorA_y.xzy);	
	float3 normalA = mul(matrixA, normal);
		
	float3 vectorB_x = float3(0.707107, -0.707107, 0);
	float3 vectorB_y = float3(0.572062, 0.572061, 0.587786);
	float3 vectorB_z = float3(-0.415627, -0.415627, 0.809017); // Z is up
	float3x3 matrixB = float3x3(vectorB_x.xzy, vectorB_z.xzy, vectorB_y.xzy);	
	float3 normalB = mul(matrixB, normal);
		
	float3 vectorC_x = float3(0.572061, 0.572062, 0.587785);
	float3 vectorC_y = float3(-0.707107,0.707107, 0);
	float3 vectorC_z = float3(-0.415627, -0.415627, 0.809017); // Z is up
	float3x3 matrixC = float3x3(vectorC_x.xzy, vectorC_z.xzy, vectorC_y.xzy);	
	float3 normalC = mul(matrixA, normal);
*/
	float mipLevel = 10;
	
	float3 cubemapA = cubeMapTexture.SampleLevel(defaultWrapSampler, normal, mipLevel).rgb;
	//float3 cubemapB = cubeMapTexture.SampleLevel(defaultWrapSampler, normalB, mipLevel).rgb;
	//float3 cubemapC = cubeMapTexture.SampleLevel(defaultWrapSampler, normalC, mipLevel).rgb;
	
	float3 blurredCubemap = cubemapA;//(cubemapA + cubemapB + cubemapC) / 3;
    return blurredCubemap * 4;
}


float GGXDistribution(float3 N, float3 H, float roughness)
{
	float a = roughness * roughness;
	float a2 = a * a;
	float NdotH = max(dot(N, H), 0.0);
	float NdotH2 = NdotH * NdotH;

	float num = a2;
	float denom = (NdotH2 * (a2 - 1.0) + 1.0);
	denom = 3.14159265358979 * denom * denom;

	return num / denom;
}

float GGXGeometrySchlick(float NdotV, float roughness)
{
	float r = (roughness + 1.0);
	float k = (r*r) / 8.0;

	float num = NdotV;
	float denom = NdotV * (1.0 - k) + k;

	return num / denom;
}
float GeometrySmith(float3 N, float3 V, float3 L, float roughness)
{
	float NdotV = max(dot(N, V), 0.0);
	float NdotL = max(dot(N, L), 0.0);
	float ggx2 = GGXGeometrySchlick(NdotV, roughness);
	float ggx1 = GGXGeometrySchlick(NdotL, roughness);

	return ggx1 * ggx2;
}

float3 Intensity(float3 aValue, float aIntensity)
{
    float a = pow(aIntensity, -2.0f);
    return pow(aValue, a.rrr);
}

// Taken from Norrsken PBR ShaderFX shader
float3 Fresnel( float3 albedo, float roughness, float metallic, float3 viewVector, float3 normalVector)
{
	float3 F0 = lerp(float3(0.04, 0.04, 0.04), albedo, metallic);
	float NdV = max(dot(normalVector, viewVector), 0);
	float3 roughComp = float3(1- roughness, 1- roughness, 1- roughness);

	return F0 + (max(roughComp, F0) - F0) * pow(1 - NdV, 5);
}

// Taken from Norrsken PBR ShaderFX shader
float polynomeExp6(float inputX, float inputA, float inputB, float inputC , float inputD, float inputE , float inputF , float inputG  ) 
{ 
	float inputX2 = inputX * inputX;
	float inputX4 = inputX2 * inputX2;

	inputB *= inputX;
	inputC *= inputX2;
	inputD *= inputX2 * inputX;
	inputE *= inputX4;
	inputF *= inputX4 * inputX;
	inputG *= inputX4 * inputX2;

	return (inputA + inputB) + (inputC + inputD) + (inputE + inputF) + inputG; 
} 
// Taken from Norrsken PBR ShaderFX shader
float NorrskenPBR_CurveFittedBRDF_X (float inputX, float inputY) // Custom curve fitting solution for Norrsken PBR
{
	float Y_0d0000 = polynomeExp6(inputX, 0.043, 	4.6407, 	-8.6198, 	7.2295, 	-2.2951,	0, 			0				);	// A
	float Y_0d1250 = polynomeExp6(inputX, 0.1312, 	3.1677, 	-3.5215, 	0.7417, 	0.4844, 	0,			0				);	// A-B filler
	float Y_0d2500 = polynomeExp6(inputX, 0.381, 	-0.7859, 	14.11, 		-34.995, 	34.255, 	-11.972, 	0				); 	// B

	float Y_0d5000 = polynomeExp6(inputX, 0.6669, 	-1.0856, 	4.6217, 	-5.4772,	2.1703, 	0, 			0				);	// C

	float Y_0d7500 = polynomeExp6(inputX, 0.6989, 	-0.7162, 	1.4551, 	-1.2672, 	0.4376, 	0, 			0				);	// D

	float Y_1d0000 = polynomeExp6(inputX, 0.5955, 	-0.5287, 	0.4386, 	-0.2757, 	0.0842, 	0, 			0				);	// E


	float rangeAB = lerp(Y_0d0000, Y_0d1250, saturate(inputY * 8));
	rangeAB = lerp(rangeAB, Y_0d2500, saturate(inputY * 8 - 1));
	float rangeCD = lerp(Y_0d5000, Y_0d7500, saturate(inputY * 4 - 2));
	float rangeAD = lerp(rangeAB, rangeCD, saturate(inputY * 4 - 1));
	float rangeAE = lerp(rangeAD, Y_1d0000, saturate(inputY * 4 - 3));

	return saturate(rangeAE);
}
// Taken from Norrsken PBR ShaderFX shader
float NorrskenPBR_CurveFittedBRDF_Y (float inputX, float inputY) // Custom curve fitting solution for Norrsken PBR
{
	float Y_0d0000 = polynomeExp6(inputX, 0.9783, 	-3.6515, 	4.4945, 	-1.7578, 	-0.0562,	0, 			0				);	// A
	float Y_0d1250 = polynomeExp6(inputX, 0.5952, 	1.3029, 	-21.359, 	64.788, 	-89.824, 	60.826, 	-16.329			);	// A-B filler
	float Y_0d2500 = polynomeExp6(inputX, 0.4503, 	-1.3568, 	8.065, 		-40.677, 	86.035, 	-78.837, 	26.32			); 	// B

	float Y_0d5000 = polynomeExp6(inputX, 0.2025, 	-0.6279, 	0.5284, 	0.0465, 	-0.1498,	0,			0				);	// C

	float Y_0d7500 = polynomeExp6(inputX, 0.0727, 	-0.4359, 	1.1113, 	-1.2707, 	0.5235,		0, 			0				);	// D

	float Y_1d0000 = polynomeExp6(inputX, 0.0277, 	-0.1821, 	0.4488, 	-0.4802, 	0.186, 		0, 			0				);	// E


	float rangeAB = lerp(Y_0d0000, Y_0d1250, saturate(inputY * 8));
	rangeAB = lerp(rangeAB, Y_0d2500, saturate(inputY * 8 - 1));
	float rangeCD = lerp(Y_0d5000, Y_0d7500, saturate(inputY * 4 - 2));
	float rangeAD = lerp(rangeAB, rangeCD, saturate(inputY * 4 - 1));
	float rangeAE = lerp(rangeAD, Y_1d0000, saturate(inputY * 4 - 3));

	return saturate(rangeAE);
}

PixelOutput main(VertexToPixel input)
{

    float3 normal = normalize(Normal(input).xzy);

	float3 view = normalize(cameraPos.xzy - WorldPos(input).xzy);
	
	
	float3 albedo = Albedo(input).rgb;
	//albedo = pow(albedo, 1 / gamma);
	
    float roughness = Roughness(input);//(Roughness(input) + 0.5) / 2;
	//roughness = pow(roughness,0.45454545);
	
	float metallic = Metalness(input);
	//metallic = pow(metallic,1 / gamma);
	
    float ambientOcclusion = AmbientOcclusion(input);
	//ambientOcclusion = pow(ambientOcclusion,1 / gamma);
	
	float3 ambientDiffuse = AmbientDiffuse(input);
		
    float3 ambientSpec = AmbientSpec(input, roughness);

    float3 emissive = Emissive(input) * albedo;

    float VdotN = saturate(dot(normal, view));
	float2 envBRDF = float2(NorrskenPBR_CurveFittedBRDF_X(VdotN, roughness), NorrskenPBR_CurveFittedBRDF_Y(VdotN, roughness));
	
	float3 fresnel = Fresnel(albedo, roughness, metallic, view, normal);

	float3 specular = ambientSpec * (fresnel * envBRDF.x + envBRDF.y);
	
	float3 kDiffuse = float3(1,1,1) - fresnel;
	kDiffuse *= 1.0 - Metalness(input);	

	
    float3 diffuse = ambientDiffuse * kDiffuse * albedo;
    //float3 ambient = (Intensity(diffuse, myDiffuseIntensity) + Intensity(specular, mySpecularIntensity)) * ambientOcclusion;
    float3 ambient = (diffuse * myDiffuseIntensity + specular * mySpecularIntensity);
	
    PixelOutput returnValue;
    /*float luminance = dot(ambient, float3(0.2126f, 0.7152f, 0.0722f));
    float3 c = ambient * luminance * 1.5;
    float pattern = input.myUV.r + input.myUV.g;
    float patternScale = 0.4f;
    pattern *= 255;
    pattern = fmod(pattern, patternScale);
    pattern = (pattern / patternScale) * 2.0 - 1.0f;
    pattern *= 0.5f;
    c.rgb *= 255.0;
    c.rgb += pattern;
    c.rgb /= 255.0;*/
    returnValue.myColor.rgb = ambient * ambientOcclusion + emissive; //pow(ambient + emissive, gamma) * 3 * float3(1,1,1);//ambient + emissive;// + ambientOcclusion * fakeLightsource(input);
    returnValue.myColor.a = 1 * IsNotEmpty(input);
    return returnValue;
}