#include "..\Globals.hlsli"
#include "..\GBufferInput.hlsli"

cbuffer LightBuffer : register(b2)
{
    struct PointLightData
    {
        float4 myPosition;
        float3 myColor;
        float myRange;
    } myPointLightData;
}

float GGXDistribution(float NdotH, float roughness)
{
	float a = roughness * roughness;
	float a2 = a * a;
	float NdotH2 = NdotH * NdotH;

	float num = a2;
	float denom = NdotH2 * (a2 - 1.0) + 1.0;
	denom = 3.14159265358979 * denom * denom;

	return num / denom;
}

PixelOutput main(VertexToPixel input)
{
	float roughness = Roughness(input);
	
	float3 normalVector = normalize(Normal(input));
	float3 viewVector = normalize(cameraPos.xyz - WorldPos(input).xyz);
    float3 lightColor = myPointLightData.myColor.rgb;
    float3 lightPosition = myPointLightData.myPosition.xyz;
	float3 lightVector = normalize(lightPosition - WorldPos(input).xyz);
	
	float3 halfVector = normalize(viewVector + lightVector);

	float distance = length(myPointLightData.myPosition.xyz - WorldPos(input).xyz);
	
    float attenuation = saturate(1.0 - distance / myPointLightData.myRange);
	attenuation = smoothstep(0.0, 0.5, attenuation) * attenuation * attenuation;
	float3 radiance = lightColor * attenuation;
	
	float NdL = max(dot(normalVector, lightVector), 0.0);
	float NdH = max(dot(normalVector, halfVector), 0.0);

    float3 cheapLight = (NdL * Albedo(input) + NdL * GGXDistribution(NdH, roughness)) * radiance; // texture reads are too expensive
	
    PixelOutput returnValue;
    returnValue.myColor.rgb = cheapLight;
    returnValue.myColor.a = 1 * IsNotEmpty(input);
    return returnValue;
}