#include "..\Globals.hlsli"
#include "..\GBufferInput.hlsli"
#include "..\SkyboxInput.hlsli"

Texture2D texture0 : register(t20);

static const int   c_samplesX = 15;  // must be odd
static const int   c_samplesY = 15;  // must be odd
static const float c_textureSize = 512.0;

static const int   c_halfSamplesX = c_samplesX / 2;
static const int   c_halfSamplesY = c_samplesY / 2;
static const float c_pixelSize = (1.0 / c_textureSize);

float Gaussian(float sigma, float x)
{
	return exp(-(x*x) / (2.0 * sigma*sigma));
}

float3 BlurredPixel(in float2 uv)
{
	float c_sigmaX = 50.0 * (sin(5.0)*0.5 + 0.5) * 5.0;
	float c_sigmaY = 50.0 * c_sigmaX;

	float total = 0.0;
	float3 ret = float(0);

	for (int iy = 0; iy < c_samplesY; ++iy)
	{
		float fy = Gaussian(c_sigmaY, float(iy) - float(c_halfSamplesY));
		float offsety = float(iy - c_halfSamplesY) * c_pixelSize;
		for (int ix = 0; ix < c_samplesX; ++ix)
		{
			float fx = Gaussian(c_sigmaX, float(ix) - float(c_halfSamplesX));
			float offsetx = float(ix - c_halfSamplesX) * c_pixelSize;
			total += fx * fy;
			ret += colorTexture.Sample(pointClampSampler, uv + float2(offsetx, offsety)).rgb * fx*fy;
		}
	}
	return ret / total;
}

PixelOutput main(VertexToPixel input)
{
    float distance = 200; //levelFogDistance; // mAYBE ENGINE VALUE ?
	float dexcity = 0; // this too
	PixelOutput output;
    float depth = depthTexture.Sample(pointClampSampler, input.myUV).r;



	float3 Col = texture0.Sample(defaultClampSampler, input.myUV).rgb;//Albedo(input).rgb;
	float3 Skybox = skyboxTexture.Sample(defaultClampSampler, input.myUV).rgb;//Albedo(input).rgb;


	depth *= 1.7f;
	if (depth < 0.0000000000000000000000000001)
	{
		discard;
	}
	depth += (depth*distance);
	depth = pow(depth, .6);
	depth = 1 - depth;
	depth = clamp(depth, 0, 1);



	
	
	float3 ColLowSat = ((Col.rgb - 0.1) * max(.1, 0)) + 0.5f;

	Col = lerp(Col, ColLowSat, depth*1.5);

	float3 fDepth = float3(depth, depth, depth);
	float3 color = float3(1.0*dexcity, 1.0*dexcity, 1.0*dexcity);
	//
	float3 depthCol = (BlurredPixel(input.myUV)*color)*Skybox;
	
	Col = lerp(Col, float3(1.0, 1.0, 1.0), depth*depth);

	output.myColor.rgb = Col;//depth;// mul(toCamera, float4(Normal(input), 1.0f));// Col;// Col.rbg *
	output.myColor.a = 1.f;
	return output;




}