#include "..\Globals.hlsli"
#include "..\GBufferInput.hlsli"

static const float g_samples = 32;
static const float g_sample_rad = 0.5;
static const float g_sample_maxRad = 0.125;
static const float g_intensity = 1;

static const float3 samples[16] = {
float3(-0.145384, 0.917884, 0.369258),
float3( 0.453986, 0.891009, 0.000000),
float3( 0.828037, 0.421904, 0.369258),
float3( 0.987688,-0.156434, 0.000000),
float3( 0.657135,-0.657132, 0.369258),
float3( 0.156430,-0.987689, 0.000000),
float3(-0.421910,-0.828034, 0.369258),
float3(-0.891009,-0.453986, 0.000000),
float3(-0.917885, 0.145379, 0.369258),
float3(-0.707111, 0.707103, 0.000000),
float3(-0.463591, 0.463585, 0.755098),
float3( 0.297639, 0.584156, 0.755098),
float3( 0.647544,-0.102561, 0.755098),
float3( 0.102557,-0.647541, 0.755098),
float3(-0.584157,-0.297639, 0.755098),
float3( 0.000000, 0.000000, 1.000000)
};

float3 dtc (float3 dir) // direction to color
{
	return 0.5 + 0.5 * dir;
}

PixelOutput main(VertexToPixel input)
{
	PixelOutput output;

	output.myColor = float(1);
	const float2 randCoords[4] = { float2(1,0),float2(-1,0),float2(0,1),float2(0,-1) };
	
	float3 position = WorldPos(input).xyz;
	position = mul(toCamera, float4(position.xyz, 1));

	float3 normal = mul(toCamera, Normal(input)).xyz;
	normal = normalize(normal);
	normal.z *= -1;

	float randX = frac(frac(input.myUV.x * 1425.512321) * input.myUV.y * 125155.2352);
	float randY = frac(frac(input.myUV.y * 3151.144563) * input.myUV.x * 272145.8152);
	
	float3 randomVec = float3(randX, randY, randX);

	float3 tangent   = normalize(cross(randomVec, normal));
	float3 bitangent = cross(normal, tangent);
	float3x3 TBN = float3x3(tangent, bitangent, normal);  
	
	float2 texelSize = 1.0 / float2(screenWidth, screenHeight);
	int kernelSize = g_samples;
	
	float radiusA = abs(fmod(412352.214315 * randomVec.x, 1));
	
	float depthBuffer = depthTexture.Sample(pointClampSampler, input.myUV).r;
	float skyboxCull = depthBuffer;
	skyboxCull = (skyboxCull == 0 ? 1 : 0);

	float patternDepthScale = 1.0;
	float sample_rad = g_sample_rad;
	
    float occlusion = 0.0;
	for(int i = 0; i < kernelSize; ++i)
    {
		float sampleScalar =  abs(fmod(1.6143314215 * (i * 4 + randomVec.x), 1));
		float3 sampleCoord = samples[fmod(i, 16)] * pow(sampleScalar, 2);
		float3 sample = mul(sampleCoord, TBN); // From tangent to view-space
		sample = float3(sample.x, sample.y, -sample.z);
		sample = float3(position.x, position.y, position.z) + sample * sample_rad; 
		
		float4 offset	= mul(toProjection, float4(sample, 1.0));		// from view to clip-space
		offset.xyz 		/= offset.w;               // perspective divide
		offset.xyz  	= offset.xyz * 0.5 + 0.5; // transform to range 0.0 - 1.0  
		
		offset.xy = float2(offset.x, 1 - offset.y);
		
		float sampleDepth = depthTexture.Sample(pointClampSampler, offset.xy).r;

		sampleDepth = LinearDepthCalc(sampleDepth);

		float diff = sample.z - sampleDepth;
		diff = (diff < 0.1 * sample_rad ? 1.0 : smoothstep(0, 1 * sample_rad * patternDepthScale, diff));
		
		occlusion += diff;

		
    }
    occlusion = occlusion / kernelSize;
	occlusion = pow(occlusion, g_intensity);
		
    output.myColor.rgb = max(occlusion, skyboxCull);
	output.myColor.a = 1.0;


	return output;



}
