#include "..\Globals.hlsli"
#include "..\GBufferInput.hlsli"

Texture2D texture0 : register(t20);

#define FXAA_SPAN_MAX 8.0
#define FXAA_REDUCE_MUL   (1.0/FXAA_SPAN_MAX)
#define FXAA_REDUCE_MIN   (1.0/128.0)
#define FXAA_SUBPIX_SHIFT (1.0/4.0)

float3 FxaaPixelShader(float4 uv,float2 rcpFrame)
{

    float3 rgbNW = texture0.SampleLevel(defaultClampSampler, uv.zw, 0).xyz;
    float3 rgbNE = texture0.SampleLevel(defaultClampSampler, uv.zw + float2(1, 0) * rcpFrame.xy, 0).xyz;
    float3 rgbSW = texture0.SampleLevel(defaultClampSampler, uv.zw + float2(0, 1) * rcpFrame.xy, 0).xyz;
    float3 rgbSE = texture0.SampleLevel(defaultClampSampler, uv.zw + float2(1, 1) * rcpFrame.xy, 0).xyz;
    float3 rgbM = texture0.SampleLevel(defaultClampSampler, uv.xy, 0).xyz;

	float3 luma = float3(0.299, 0.587, 0.114);
	float lumaNW = dot(rgbNW, luma);
	float lumaNE = dot(rgbNE, luma);
	float lumaSW = dot(rgbSW, luma);
	float lumaSE = dot(rgbSE, luma);
	float lumaM = dot(rgbM, luma);

	float lumaMin = min(lumaM, min(min(lumaNW, lumaNE), min(lumaSW, lumaSE)));
	float lumaMax = max(lumaM, max(max(lumaNW, lumaNE), max(lumaSW, lumaSE)));

	float2 dir;
	dir.x = -((lumaNW + lumaNE) - (lumaSW + lumaSE));
	dir.y = ((lumaNW + lumaSW) - (lumaNE + lumaSE));

	float dirReduce = max(
		(lumaNW + lumaNE + lumaSW + lumaSE) * (0.25 * FXAA_REDUCE_MUL),
		FXAA_REDUCE_MIN);
	float rcpDirMin = 1.0 / (min(abs(dir.x), abs(dir.y)) + dirReduce);

	dir = min(float2(FXAA_SPAN_MAX, FXAA_SPAN_MAX),
		max(float2(-FXAA_SPAN_MAX, -FXAA_SPAN_MAX),
			dir * rcpDirMin)) * rcpFrame.xy;

    float3 rgbA = (1.0 / 2.0) * (texture0.SampleLevel(defaultClampSampler, uv.xy + dir * (1.0 / 3.0 - 0.5), 0.0).xyz +
		texture0.SampleLevel(defaultClampSampler, uv.xy + dir * (2.0 / 3.0 - 0.5), 0.0).xyz);
    float3 rgbB = rgbA * (1.0 / 2.0) + (1.0 / 4.0) * (texture0.SampleLevel(defaultClampSampler, uv.xy + dir * (0.0 / 3.0 - 0.5), 0.0).xyz +
		texture0.SampleLevel(defaultClampSampler, uv.xy + dir * (3.0 / 3.0 - 0.5), 0.0).xyz);

	float lumaB = dot(rgbB, luma);

	if ((lumaB < lumaMin) || (lumaB > lumaMax))
	{
		return rgbA;
	}
	else
	{
		return rgbB;
	}
}

PixelOutput main(VertexToPixel input)
{
	PixelOutput output;
	float2 res = float2(screenWidth, screenHeight);
	float2 rcpFrame = 1.f / res.xy;
	float2 uv2 = input.myPosition.xy;
	
	float4 uv = float4(input.myUV.xy, input.myUV.xy - (rcpFrame * (0.5 + FXAA_SUBPIX_SHIFT)));

	output.myColor.rgb = FxaaPixelShader(uv, rcpFrame.xy).rgb;
	
	output.myColor.a= 1.0f;

	return output;
}