#include "..\Globals.hlsli"
#include "..\GBufferInput.hlsli"

Texture2D texture0 : register(t20);

float extractLuma(float3 c)
{
	return c.r * 0.299 + c.g * 0.587 + c.b * 0.114;
}
static float3x3 YUVFromRGB = float3x3(
	float3(0.299, 0.587, 0.114),
	float3(-0.14713, -0.28886, 0.436),
	float3(0.615, -0.51499, -0.10001));

static float3x3 RGBFromYUV = float3x3(
	float3(1, 0, 1.13983),
	float3(1, -0.394, -0.58),
	float3(1, 2.03211, 0.0));

PixelOutput main(VertexToPixel input)
{
	PixelOutput output;


    float4 s = texture0.Sample(defaultClampSampler, input.myUV);

	float3 yuv = mul(YUVFromRGB, s.rgb);

	float2 imgSize = float2(1024, 1024);//sharpness

	float accumY = 0.0;
	for (int i = -1; i <= 1; ++i)
	{
		for (int j = -1; j <= 1; ++j)
		{
			float2 offset = float2(i, j) / imgSize;

            float s = extractLuma(texture0.Sample(defaultClampSampler, input.myUV + offset).rgb);
			float notCentre = min(float(i*i + j * j), 1.0);
			accumY += s * (9.0 - notCentre * 10.0);
		}
	}

	accumY /= 9.0;

	float gain = 0.9;
	accumY = (accumY + yuv.x)*gain;

	output.myColor = float4(mul(RGBFromYUV, float3(accumY, yuv.y, yuv.z)), 1.0); // sharpened


	return output;
}