#include "..\Globals.hlsli"
#include "..\ModelShaderStructs.hlsli"

cbuffer ObjectBuffer : register(b1)
{
    float4x4 toWorld;
};

cbuffer BoneBuffer : register(b2)
{
    float4x4 Bones[64];
};

VertexToPixel main(VertexInputAnimation input)
{
    VertexToPixel result;

    float4 objectPos = input.myPosition.xyzw;
    //objectPos.xz = -objectPos.xz;

    float4 weights = input.myBoneWeights;
    uint4 boneIDs = uint4((uint) input.myBoneIDs.x, (uint) input.myBoneIDs.y, (uint) input.myBoneIDs.z, (uint) input.myBoneIDs.w);
	
    float4x4 finalMatrix;
    finalMatrix = mul(Bones[boneIDs.x], weights.x);
    finalMatrix += mul(Bones[boneIDs.y], weights.y);
    finalMatrix += mul(Bones[boneIDs.z], weights.z);
    finalMatrix += mul(Bones[boneIDs.w], weights.w);

    float4 norm = input.myNormal;
    //norm.xz = -norm.xz;
    float4 tan = input.myTangent;
    //tan.xz = -tan.xz;
    float4 bi = input.myBinormal;
    //bi.xz = -bi.xz;

    float4 animatedPos = mul(finalMatrix, objectPos);
    float4 animatedNorm = mul(finalMatrix, float4(norm.xyz, 0));
    float4 animatedTan = mul(finalMatrix, float4(tan.xyz, 0));
    float4 animatedBi = mul(finalMatrix, float4(bi.xyz, 0));

    float4 worldPos = mul(toWorld, animatedPos);
    float4 viewPos = mul(toCamera, worldPos);
    float4 projectionPos = mul(toProjection, viewPos);

    float3x3 toWorldRotation = (float3x3) toWorld;
    float3 normal = mul(toWorldRotation, animatedNorm.xyz);
    float3 tangent = mul(toWorldRotation, animatedTan.xyz);
    float3 binormal = mul(toWorldRotation, animatedBi.xyz);

    result.myPosition = projectionPos;
    result.myWorldPosition = worldPos;
    result.myNormal = float4(normal, 0);
    result.myTangent = float4(tangent, 0);
    result.myBinormal = float4(binormal, 0);
    result.myColor = float4(1, 1, 1, 1);
    result.myUV = input.myUV;

    return result;
}